!MNH_LIC Copyright 1994-2024 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
!####################################
MODULE MODI_READ_PRE_IDEA_NAM_n
!####################################
INTERFACE 
  SUBROUTINE READ_PRE_IDEA_NAM_n( TPFILEPRE )
    USE MODD_IO,        ONLY: TFILEDATA

    TYPE(TFILEDATA), INTENT(IN) :: TPFILEPRE ! EXPRE file
  END SUBROUTINE READ_PRE_IDEA_NAM_n
END INTERFACE
!
END MODULE MODI_READ_PRE_IDEA_NAM_n
!
!
!     ###########################################
      SUBROUTINE READ_PRE_IDEA_NAM_n( TPFILEPRE )
!     ###########################################

USE MODD_CH_AEROSOL, ONLY: LORILAM
USE MODD_CH_MNHC_n,  ONLY: LCH_INIT_FIELD_n=>LCH_INIT_FIELD, &
                           CCHEM_INPUT_FILE_n=>CCHEM_INPUT_FILE
USE MODD_DIM_n,      ONLY: NIMAX_n=>NIMAX, NJMAX_n=>NJMAX
USE MODD_DYN_n,      ONLY: CPRESOPT_n=>CPRESOPT,NITR_n=>NITR,XRELAX_n=>XRELAX, &
                           LRES_n=>LRES,XRES_n=>XRES
USE MODD_IO,         ONLY: TFILEDATA
USE MODD_LBC_n,      ONLY: CLBCX_n=>CLBCX,CLBCY_n=>CLBCY
USE MODD_PARAM_n,    ONLY: CSURF_n=>CSURF
USE MODD_PARAMETERS, ONLY: NFILENAMELGTMAX

USE MODE_POS

USE MODN_CONF_n
USE MODN_LUNIT_n           ! Namelist modules

IMPLICIT NONE

TYPE(TFILEDATA), INTENT(IN) :: TPFILEPRE ! EXPRE file

LOGICAL :: GFOUND                  ! Return code when searching namelist 
! Namelist variables from $n
INTEGER                :: NIMAX,NJMAX
CHARACTER(LEN=5)       :: CPRESOPT
INTEGER                :: NITR
LOGICAL                :: LRES
REAL                   :: XRES
REAL                   :: XRELAX      
CHARACTER(LEN=4), DIMENSION(2) :: CLBCX,CLBCY
CHARACTER(LEN=5)       :: CSURF
LOGICAL                :: LCH_INIT_FIELD
CHARACTER(LEN=NFILENAMELGTMAX) :: CCHEM_INPUT_FILE

INTEGER                :: ILUPRE
!
!*       0.2  Namelist declarations
!
NAMELIST/NAM_DIMn_PRE/ NIMAX,NJMAX          ! Declaration in MODD_DIM1
NAMELIST/NAM_DYNn_PRE/ CPRESOPT,NITR,XRELAX,LRES,XRES ! Declaration in MODD_DYN1
NAMELIST/NAM_LBCn_PRE/ CLBCX,CLBCY          ! Declaration in MODD_LBC1
NAMELIST/NAM_GRn_PRE/  CSURF                ! Declaration in MODD_PARAM1
NAMELIST/NAM_CH_MNHCn_PRE/ LCH_INIT_FIELD, CCHEM_INPUT_FILE, LORILAM
!
!------------------------------------------------------------------------------
!
ILUPRE = TPFILEPRE%NLU

CALL INIT_NMLVAR
CALL POSNAM( TPFILEPRE, 'NAM_DIMN_PRE', GFOUND )
IF (GFOUND) READ(UNIT=ILUPRE,NML=NAM_DIMn_PRE)
CALL POSNAM( TPFILEPRE, 'NAM_DYNN_PRE', GFOUND )
IF (GFOUND) READ(UNIT=ILUPRE,NML=NAM_DYNn_PRE)
CALL POSNAM( TPFILEPRE, 'NAM_LBCN_PRE', GFOUND )
IF (GFOUND) READ(UNIT=ILUPRE,NML=NAM_LBCn_PRE)
CALL POSNAM( TPFILEPRE, 'NAM_GRN_PRE', GFOUND )
IF (GFOUND) READ(UNIT=ILUPRE,NML=NAM_GRn_PRE)
CALL POSNAM( TPFILEPRE, 'NAM_CH_MNHCN_PRE', GFOUND )
IF (GFOUND) READ(UNIT=ILUPRE,NML=NAM_CH_MNHCn_PRE)
CALL UPDATE_MODD_FROM_NMLVAR
!
CALL POSNAM( TPFILEPRE, 'NAM_CONFN', GFOUND )
IF (GFOUND) THEN 
  CALL INIT_NAM_CONFn
  READ(UNIT=ILUPRE,NML=NAM_CONFn)
  CALL UPDATE_NAM_CONFn
END IF
CALL POSNAM( TPFILEPRE, 'NAM_LUNITN', GFOUND )
IF (GFOUND) THEN 
  CALL INIT_NAM_LUNITn 
  READ(UNIT=ILUPRE,NML=NAM_LUNITn)
  CALL UPDATE_NAM_LUNITn(1)
END IF

!------------------------------------------------------------------------------
CONTAINS

SUBROUTINE INIT_NMLVAR

NIMAX=NIMAX_n
NJMAX=NJMAX_n
CPRESOPT=CPRESOPT_n
LRES=LRES_n
XRES=XRES_n
NITR=NITR_n
XRELAX=XRELAX_n
CLBCX=CLBCX_n
CLBCY=CLBCY_n
CSURF=CSURF_n
LCH_INIT_FIELD=LCH_INIT_FIELD_n
CCHEM_INPUT_FILE=CCHEM_INPUT_FILE_n
END SUBROUTINE INIT_NMLVAR

SUBROUTINE UPDATE_MODD_FROM_NMLVAR

NIMAX_n=NIMAX
NJMAX_n=NJMAX
CPRESOPT_n=CPRESOPT
LRES_n=LRES
XRES_n=XRES
NITR_n=NITR
XRELAX_n=XRELAX
CLBCX_n=CLBCX
CLBCY_n=CLBCY
CSURF_n=CSURF
LCH_INIT_FIELD_n=LCH_INIT_FIELD
CCHEM_INPUT_FILE_n=CCHEM_INPUT_FILE
END SUBROUTINE UPDATE_MODD_FROM_NMLVAR

END SUBROUTINE READ_PRE_IDEA_NAM_n
